#include "Arduino.h"
#include "TeensyStep.h"
#include "ADNS2610.h"
#include <Encoder.h>

//Encoder variables
#define  A_PHASE 14 // green cable
#define  B_PHASE 2 // white cable

const int gear_ratio = 6;

//Encoder object
Encoder encoder(A_PHASE, B_PHASE);

float ppr = gear_ratio*2458; //2458 is the number of pulses per encoder revolution
float deg_pulse = 360/ppr;
float encoder_angle = 0;

//Stepper configuration
const int steps_rev = 25000*gear_ratio;//6 times 25000 steps/revolution

const int omega_deg = 20; // Angular velocity in deg*s^-1
const int alpha_deg = 200; // Angular acceleration in deg*s^-2

const int omega_steps = (omega_deg*steps_rev)/360; // Angular velocity in steps*s^-1
const int alpha_steps = (alpha_deg*steps_rev)/360; // Angular acceleration in steps*s^-2

float  desired_position = 0;
float  actual_position = 0;

//Optical sensor variables
#define SCLK1 8                            // Serial clock pin
//#define SCLK1 3                            // Serial clock pin
#define SDIO1 7                           // Serial data (I/O) pin

// Create an instance of the ADNS2610 object for each mouse
ADNS2610 Optical1 = ADNS2610(SCLK1, SDIO1);
                
int SQUAL;

// Cosntants and conversion factors
const float pi = 3.14159265;
const float radius = 9.85;

float conversion_factor = 151.81; //151.81 optical sensor units/1cm
float cm_to_deg = 360/(2*pi*radius);

signed long y_cum = 0;

float a;
float angle;
const float steps_sensor_internal_units = 15.97;

constexpr unsigned P_Interval = 10; // ms  
constexpr float kp = 0.01;             // (P)roportional constant of the regulator needs to be adjusted (depends on speed and acceleration setting)

Stepper stepper(5, 6); // STEP at pin 5, DIR at pin 6
RotateControl controller;

int32_t Setpoint = 0;

int hallSensorPin = 4;

int DAC_out_0;
int DAC_out_1;

void Homing()
{
  while (digitalRead(hallSensorPin) == HIGH)
  {
    TurnCounterClockwise();
  }
  delay(1000);
}

void TurnCounterClockwise ()
{
   // If you decrease the delay, the homing velocity will increase
   digitalWrite(6,1);
   digitalWrite(5,HIGH);
   delayMicroseconds(500);
   digitalWrite(5,LOW);
   delayMicroseconds(500);
}

void Proportional_control()
{
  static unsigned lastTick = 0;

  if (millis() - lastTick > P_Interval)
  {    
    float delta = (Setpoint - stepper.getPosition()) * (kp / P_Interval);  // This implements a simple P regulator (can be extended to a PID if necessary)
    float factor = std::max(-1.0f, std::min(1.0f, delta)); // limit to -1.0..1.0

    controller.overrideSpeed(factor); // set new speed
    lastTick = 0;
  }
}

void setup()
{
  pinMode(hallSensorPin, INPUT);

  analogWriteResolution(12);
  
  Optical1.begin(); 

  Serial.begin(115200);

  Homing();
  
  stepper
      .setMaxSpeed(omega_steps) 
      .setAcceleration(alpha_steps); 

  controller.rotateAsync(stepper);
  controller.overrideSpeed(0); // start with stopped slide

  //Encoder variable to zero
  encoder.write(0);

}

elapsedMillis outputTimer = 0;

void loop()
{
    
  //SQUAL = Optical1.squal();

  Proportional_control();

  y_cum += Optical1.dy();
  a = y_cum*steps_sensor_internal_units;//With the configuration of 25000 steps per rev on the stepper 6 times 150000 ppr (gear ratio) for each mouse unit, the motor has to move 2.09 steps making a 0.09 steps of error
  Setpoint = -1*int(a);

  desired_position = float((y_cum)*(1/conversion_factor))*cm_to_deg;
  actual_position =(float(stepper.getPosition())*360)/steps_rev;

  encoder_angle = float(encoder.read()*deg_pulse*-1);

  DAC_out_0 = int(5.6875*desired_position + 2047.5);
  DAC_out_1 = int(5.6875*(actual_position*-1) + 2047.5);
  
  analogWrite(A21, DAC_out_0);
  analogWrite(A22, DAC_out_1);
  
  //print to the serial port
  
  //  Serial.print(SQUAL);
  //  Serial.print("    ");
   // Serial.print(desired_position);
    Serial.print(encoder_angle);
    Serial.print("  ");
    Serial.println((actual_position*-1));
  //  Serial.print("    ");
  //  Serial.print(millis());
  //  Serial.print("    ");
  //  Serial.println(micros()-time_);

}
